# all this was necessary to give me a case sensitive predicate!!!
# stolen from https://github.com/activerecord-hackery/ransack/issues/699
# {{
Ransack.configure do |config|
  config.add_predicate(
    'like',
    arel_predicate: 's_matches',
    formatter: proc { |v| "%#{v}%" },
    type: :string
  )
end

module Arel
  module Nodes
    # creates an arel predicate for case sensitive searching in postgres
    class Matches < Binary
      attr_reader :escape
      attr_accessor :case_sensitive

      def initialize(left, right, escape = nil, case_sensitive = false)
        super(left, right)
        @escape = escape && Nodes.build_quoted(escape)
        @case_sensitive = case_sensitive
      end
    end

    # DoesNotMatch will inherit from matches
    class DoesNotMatch < Matches; end
  end
end

module Arel
  # adding s_metches to Arel::Predications
  module Predications
    # creates an arel predicate for case sensitive searching in postgres
    def s_matches(other, escape = nil)
      Nodes::Matches.new self, quoted_node(other), escape, true
    end
  end
end

module Arel
  module Visitors
    # changes postgres query to casesensitive
    class PostgreSQL < Arel::Visitors::ToSql
      private

      # arel doesnt use snakecase??
      def visit_Arel_Nodes_Matches(oth, col)
        col = infix_value oth, col, oth.case_sensitive ? ' LIKE ' : ' ILIKE '
        escape = oth.escape
        return col unless escape
        visit escape, col << ' ESCAPE '
      end
    end
  end
end
# }}

# Array Contains
#  {{{
# https://github.com/activerecord-hackery/ransack/issues/321
module Arel
  # create arel predicate for array
  module Predications
    def array_cont(other)
      Nodes::Equality.new(
        Nodes.build_quoted(other, self), Nodes::NamedFunction.new('ANY', [self])
      )
    end
  end
end

Ransack.configure do |config|
  config.add_predicate :array_cont, arel_predicate: :array_cont
end
# }}}

# Array Not Contains
#  {{{
# https://github.com/activerecord-hackery/ransack/issues/321
module Arel
  # create arel predicate for not array not contains
  module Predications
    def array_not_cont(other)
      Nodes::DoesNotMatch.new(
        Nodes.build_quoted(other, self),
        Nodes::NamedFunction.new('ALL', [self])
      )
    end
  end
end

Ransack.configure do |config|
  config.add_predicate :array_not_cont, arel_predicate: :array_not_cont
end
# }}}

#
# vim:foldmethod=marker
